plugins {
    id("com.android.application")
    id("org.jetbrains.kotlin.android")
    id("com.dfsek.feldspar.compose-fixer")
}

android {
    namespace = "com.dfsek.feldspar"
    compileSdk = 33

    defaultConfig {
        applicationId = "com.dfsek.feldspar"
        minSdk = 24
        targetSdk = 33
        versionCode = 1
        versionName = "1.0"

        testInstrumentationRunner = "androidx.test.runner.AndroidJUnitRunner"
    }

    buildFeatures {
        compose = true
    }

    composeOptions {
        kotlinCompilerExtensionVersion = "1.3.2"
    }

    buildTypes {
        release {
            isMinifyEnabled = true
            proguardFiles(getDefaultProguardFile("proguard-android-optimize.txt"), "proguard-rules.pro")
        }
    }
    compileOptions {
        sourceCompatibility = JavaVersion.VERSION_1_8
        targetCompatibility = JavaVersion.VERSION_1_8
    }
    kotlinOptions {
        jvmTarget = "11"
    }
    buildFeatures {
        viewBinding = true
    }
}

dependencies {
    implementation("androidx.core:core-ktx:1.9.0")
    implementation("androidx.appcompat:appcompat:1.5.1")
    implementation("com.google.android.material:material:1.7.0")
    implementation("androidx.constraintlayout:constraintlayout:2.1.4")
    implementation("androidx.navigation:navigation-fragment-ktx:2.5.3")
    implementation("androidx.navigation:navigation-ui-ktx:2.5.3")
    implementation("androidx.preference:preference-ktx:1.2.0")
    implementation(project(":vector:multipicker"))
    val room_version = "2.4.3"

    implementation("androidx.room:room-runtime:$room_version")
    annotationProcessor("androidx.room:room-compiler:$room_version")

    // optional - Kotlin Extensions and Coroutines support for Room
    implementation("androidx.room:room-ktx:$room_version")


    implementation("org.matrix.android:matrix-android-sdk2:1.5.18") {
        exclude("com.atlassian.commonmark")
    }

    val richtextVersion = "0.16.0"

    implementation("com.halilibo.compose-richtext:richtext-ui-material:$richtextVersion")
    implementation("com.halilibo.compose-richtext:richtext-commonmark:$richtextVersion")


    implementation("io.ktor:ktor-client-android:2.2.1")
    implementation("androidx.browser:browser:1.4.0")
    implementation("io.arrow-kt:arrow-core:1.1.3")
    implementation("io.coil-kt:coil-compose:2.2.2")

    val composeBom = platform("androidx.compose:compose-bom:2022.12.00")
    implementation(composeBom)
    androidTestImplementation(composeBom)

    implementation("androidx.compose.material:material-icons-extended")

    implementation("androidx.compose.material:material")

    implementation("androidx.lifecycle:lifecycle-viewmodel-compose:2.5.1")
    implementation("androidx.activity:activity-compose:1.6.1")
    implementation("com.github.lincollincol:compose-audiowaveform:1.0.1")
    implementation ("com.github.lincollincol:amplituda:2.2.2")

    // Android Studio Preview support
    implementation("androidx.compose.ui:ui-tooling-preview")
    debugImplementation("androidx.compose.ui:ui-tooling")

    // UI Tests
    androidTestImplementation("androidx.compose.ui:ui-test-junit4")
    debugImplementation("androidx.compose.ui:ui-test-manifest")

    testImplementation("junit:junit:4.13.2")
    androidTestImplementation("androidx.test.ext:junit:1.1.5")
    androidTestImplementation("androidx.test.espresso:espresso-core:3.5.1")
}